import sys
if "--no-pylau" in sys.argv:
    sys.argv.remove("--no-pylau")
    raise ImportError

# - We will only be imported once per interpreter
#   This means we ALWAYS check for an existing instance
#   of pylau and run one if none is found

import socket
import sys
import os

# - Create the pylau socket dir if required
sockdir = os.path.join("/tmp",".pylau")
if not os.access(sockdir, 0):
    os.mkdir(sockdir)

# - Attempt to connect to an existing launcher
conn = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
try:
    pipename = os.path.join(sockdir, os.environ["DISPLAY"])
except:
    sys.stdout.write("pylau:no display")
    raise ImportError

try:
    # - Connect to pylau to launch the host script
    conn.connect(pipename)
    sys.stdout.write("pylau:connected to %s" % pipename)

    try:
        sys.argv[0] = os.path.abspath(sys.argv[0])
        conn.send(repr(sys.argv)+"\n")
        conn.send(repr(os.environ)+"\n")
        f=conn.makefile()
        if eval(f.readline()):
            sys.stdout.write("pylau:launched OK")
            sys.exit()
    except socket.error:
        sys.stdout.write("pylau:error launching")
        raise ImportError

except socket.error:
    # - There is no existing pylau instance - start one

    # - Kill an existing, duff, socket
    try: os.unlink(pipename)
    except: pass

    # - Open the socket to listen for requests on
    conn = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    conn.bind(pipename)
    conn.listen(5)

    # - Import Gtk
    import gtk

    def _stderr(text):
        sys.stdout.write("pylau:%s\n" % text)

    _test_env = ["USERNAME", "ENV", "PYTHONPATH", "HOSTTYPE",
                 "LOGNAME", "USER", "INPUTRC", "HOME", "PATH",
                 "LD_LIBRARY_PATH", "DISPLAY", "HOSTNAME"]

    # - Internal functions to launch apps correctly(!)
    def _launch(conn, info):
        try:
            sock = conn.accept()[0]
            file = sock.makefile()
            argv = eval(file.readline())
            environ = eval(file.readline())
            for key in _test_env:
                if ((environ.has_key(key) and os.environ.has_key(key)) and
                    (environ[key] != os.environ[key])):
                    sock.send(repr(0)+"\n")
                    return
            _stderr("launch %s" % str(argv))
        except:
            _stderr("error reading request")
            sock.send(repr(0)+"\n")
            return

        try:
            import imp
            path, name = os.path.split(argv[0])
            name, ext = os.path.splitext(name)
            if path: paths = [path]
            else: paths = string.split(os.environ["PATH"],":")
            fd, pathname, desc = imp.find_module(name, paths)
            app = imp.load_module(name, fd, pathname, desc)
            try:
                app.Main(argv)
            except:
                _stderr("pylau:script %s has no Main(argv)" % name)
                sock.send(repr(0)+"\n")
                return
        except:
            _stderr("unable to launch:%s:%s" % (sys.exc_info()[0], sys.exc_info()[1]))
            sock.send(repr(0)+"\n")
            return

        sock.send(repr(1)+"\n")

    # *** Dodgy GTK hacking
    def _events_pending():
        if _gtk_mainloop_count: return 0
        return _gtk_events_pending()
    def _mainiteration():
        if _gtk_mainloop_count: return 0
        return _gtk_mainiteration()

    # *** Safer GTK hacking
    def _mainloop():
        global _gtk_mainloop_count
        old = _gtk_mainloop_count
        _gtk_mainloop_count = _gtk_mainloop_count + 1
        if not old: _gtk_mainloop()

    def _mainquit(*args):
        global _gtk_mainloop_count
        _gtk_mainloop_count = _gtk_mainloop_count - 1
        if not _gtk_mainloop_count: _gtk_mainquit()

    # - Rewire gtk... !!!
    _gtk_mainloop_count = 0
    _gtk_mainloop = gtk.mainloop
    _gtk_mainquit = gtk.mainquit

    _gtk_mainiteration=gtk.mainiteration
    _gtk_events_pending=gtk.events_pending
    gtk.mainiteration=_mainiteration
    gtk.events_pending=_events_pending

    gtk.mainloop = _mainloop
    gtk.mainquit = _mainquit

    # - Hook us in to Gtk's select() calls
    gtk.input_add(conn, gtk.GDK.INPUT_READ, _launch)
